# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
from tempest.common import utils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin.common import tempest_fixtures as fixtures
from neutron_tempest_plugin.services.bgp import bgp_client


CONF = config.CONF


class BgpSpeakerTestJSONBase(base.BaseAdminNetworkTest):
    credentials = ['primary', 'admin']

    default_bgp_speaker_args = {'local_as': '1234',
                                'ip_version': 4,
                                'name': 'my-bgp-speaker',
                                'advertise_floating_ip_host_routes': True,
                                'advertise_tenant_networks': True}
    default_bgp_peer_args = {'remote_as': '4321',
                             'name': 'my-bgp-peer',
                             'peer_ip': '192.168.1.1',
                             'auth_type': 'md5', 'password': 'my-secret'}

    def setUp(self):
        self.addCleanup(self.resource_cleanup)
        super().setUp()

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.bgp_client = cls.os_primary.bgp_client
        cls.bgp_adm_client = cls.os_admin.bgp_client

    @classmethod
    def get_client_manager(cls, credential_type=None, roles=None,
                           force_new=None):
        manager = super().get_client_manager(
            credential_type=credential_type,
            roles=roles,
            force_new=force_new
        )
        return bgp_client.Manager(manager.credentials)

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        if not utils.is_extension_enabled('bgp', 'network'):
            msg = "BGP Speaker extension is not enabled."
            raise cls.skipException(msg)

        cls.admin_routerports = []
        cls.admin_floatingips = []
        cls.admin_routers = []
        cls.ext_net_id = CONF.network.public_network_id

    @classmethod
    def resource_cleanup(cls):
        for floatingip in cls.admin_floatingips:
            cls._try_delete_resource(cls.admin_client.delete_floatingip,
                                     floatingip['id'])
        for routerport in cls.admin_routerports:
            cls._try_delete_resource(
                      cls.admin_client.remove_router_interface_with_subnet_id,
                      routerport['router_id'], routerport['subnet_id'])
        for router in cls.admin_routers:
            cls._try_delete_resource(cls.admin_client.delete_router,
                                     router['id'])
        super().resource_cleanup()

    def create_bgp_speaker(self, auto_delete=True, **args):
        data = {'bgp_speaker': args}
        bgp_speaker = self.bgp_adm_client.create_bgp_speaker(data)
        bgp_speaker_id = bgp_speaker['bgp_speaker']['id']
        if auto_delete:
            self.addCleanup(self.delete_bgp_speaker, bgp_speaker_id)
        return bgp_speaker['bgp_speaker']

    def create_bgp_peer(self, **args):
        bgp_peer = self.bgp_adm_client.create_bgp_peer({'bgp_peer': args})
        bgp_peer_id = bgp_peer['bgp_peer']['id']
        self.addCleanup(self.delete_bgp_peer, bgp_peer_id)
        return bgp_peer['bgp_peer']

    def update_bgp_speaker(self, id, **args):
        data = {'bgp_speaker': args}
        return self.bgp_adm_client.update_bgp_speaker(id, data)

    def delete_bgp_speaker(self, id):
        return self.bgp_adm_client.delete_bgp_speaker(id)

    def get_bgp_speaker(self, id):
        return self.bgp_adm_client.get_bgp_speaker(id)['bgp_speaker']

    def create_bgp_speaker_and_peer(self):
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_peer = self.create_bgp_peer(**self.default_bgp_peer_args)
        return (bgp_speaker, bgp_peer)

    def delete_bgp_peer(self, id):
        return self.bgp_adm_client.delete_bgp_peer(id)

    def add_bgp_peer(self, bgp_speaker_id, bgp_peer_id):
        return self.bgp_adm_client.add_bgp_peer_with_id(bgp_speaker_id,
                                                        bgp_peer_id)

    def remove_bgp_peer(self, bgp_speaker_id, bgp_peer_id):
        return self.bgp_adm_client.remove_bgp_peer_with_id(bgp_speaker_id,
                                                           bgp_peer_id)

    def delete_address_scope(self, id):
        return self.admin_client.delete_address_scope(id)

    def create_router(self, ext_gw_info):
        try:
            router = self.admin_client.create_router(
                                            'my-router',
                                            external_gateway_info=ext_gw_info,
                                            distributed=False)
        # Retry if not running with DVR enabled
        except lib_exc.BadRequest:
            router = self.admin_client.create_router(
                                            'my-router',
                                            external_gateway_info=ext_gw_info)
        return router['router']


class BgpSpeakerTestJSON(BgpSpeakerTestJSONBase):

    """Tests the following operations in the Neutron API using the REST client:

        Create bgp-speaker
        Delete bgp-speaker
        Create bgp-peer
        Update bgp-peer
        Delete bgp-peer
    """

    @decorators.idempotent_id('df259771-7104-4ffa-b77f-bd183600d7f9')
    def test_delete_bgp_speaker(self):
        bgp_speaker = self.create_bgp_speaker(auto_delete=False,
                                              **self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.delete_bgp_speaker(bgp_speaker_id)
        self.assertRaises(lib_exc.NotFound,
                          self.get_bgp_speaker,
                          bgp_speaker_id)

    @decorators.idempotent_id('81d9dc45-19f8-4c6e-88b8-401d965cd1b0')
    def test_create_bgp_peer(self):
        self.create_bgp_peer(**self.default_bgp_peer_args)

    @decorators.idempotent_id('6ade0319-1ee2-493c-ac4b-5eb230ff3a77')
    def test_add_bgp_peer(self):
        bgp_speaker, bgp_peer = self.create_bgp_speaker_and_peer()
        bgp_speaker_id = bgp_speaker['id']
        bgp_peer_id = bgp_peer['id']

        self.add_bgp_peer(bgp_speaker_id, bgp_peer_id)
        bgp_speaker = self.get_bgp_speaker(bgp_speaker_id)
        bgp_peers_list = bgp_speaker['peers']
        self.assertEqual(1, len(bgp_peers_list))
        self.assertIn(bgp_peer_id, bgp_peers_list)

    @decorators.idempotent_id('f9737708-1d79-440b-8350-779f97d882ee')
    def test_remove_bgp_peer(self):
        bgp_peer = self.create_bgp_peer(**self.default_bgp_peer_args)
        bgp_peer_id = bgp_peer['id']
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.add_bgp_peer(bgp_speaker_id, bgp_peer_id)
        bgp_speaker = self.get_bgp_speaker(bgp_speaker_id)
        bgp_peers_list = bgp_speaker['peers']
        self.assertIn(bgp_peer_id, bgp_peers_list)

        bgp_speaker = self.remove_bgp_peer(bgp_speaker_id, bgp_peer_id)
        bgp_speaker = self.get_bgp_speaker(bgp_speaker_id)
        bgp_peers_list = bgp_speaker['peers']
        self.assertTrue(not bgp_peers_list)

    @decorators.idempotent_id('23c8eb37-d10d-4f43-b2e7-6542cb6a4405')
    def test_add_gateway_network(self):
        self.useFixture(fixtures.LockFixture('gateway_network_binding'))
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']

        self.bgp_adm_client.add_bgp_gateway_network(bgp_speaker_id,
                                                    self.ext_net_id)
        bgp_speaker = self.get_bgp_speaker(bgp_speaker_id)
        network_list = bgp_speaker['networks']
        self.assertEqual(1, len(network_list))
        self.assertIn(self.ext_net_id, network_list)

    @decorators.idempotent_id('6cfc7137-0d99-4a3d-826c-9d1a3a1767b0')
    def test_remove_gateway_network(self):
        self.useFixture(fixtures.LockFixture('gateway_network_binding'))
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.bgp_adm_client.add_bgp_gateway_network(bgp_speaker_id,
                                                    self.ext_net_id)
        bgp_speaker = self.get_bgp_speaker(bgp_speaker_id)
        networks = bgp_speaker['networks']

        self.assertIn(self.ext_net_id, networks)
        self.bgp_adm_client.remove_bgp_gateway_network(bgp_speaker_id,
                                                       self.ext_net_id)
        bgp_speaker = self.get_bgp_speaker(bgp_speaker_id)
        network_list = bgp_speaker['networks']
        self.assertTrue(not network_list)

    @decorators.idempotent_id('5bef22ad-5e70-4f7b-937a-dc1944642996')
    def test_get_advertised_routes_null_address_scope(self):
        self.useFixture(fixtures.LockFixture('gateway_network_binding'))
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.bgp_adm_client.add_bgp_gateway_network(bgp_speaker_id,
                                                  self.ext_net_id)
        routes = self.bgp_adm_client.get_bgp_advertised_routes(bgp_speaker_id)
        self.assertEqual(0, len(routes['advertised_routes']))

    @decorators.idempotent_id('cae9cdb1-ad65-423c-9604-d4cd0073616e')
    def test_get_advertised_routes_floating_ips(self):
        self.useFixture(fixtures.LockFixture('gateway_network_binding'))
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.bgp_adm_client.add_bgp_gateway_network(bgp_speaker_id,
                                                    self.ext_net_id)
        tenant_net = self.create_network()
        tenant_subnet = self.create_subnet(tenant_net)
        ext_gw_info = {'network_id': self.ext_net_id}
        router = self.create_router(ext_gw_info)
        self.admin_routers.append(router)
        self.admin_client.add_router_interface_with_subnet_id(
                                                       router['id'],
                                                       tenant_subnet['id'])
        self.admin_routerports.append({'router_id': router['id'],
                                       'subnet_id': tenant_subnet['id']})
        tenant_port = self.create_port(tenant_net)
        floatingip = self.create_floatingip(self.ext_net_id)
        self.admin_floatingips.append(floatingip)
        self.client.update_floatingip(floatingip['id'],
                                      port_id=tenant_port['id'])
        routes = self.bgp_adm_client.get_bgp_advertised_routes(bgp_speaker_id)
        self.assertEqual(1, len(routes['advertised_routes']))
        self.assertEqual(floatingip['floating_ip_address'] + '/32',
                         routes['advertised_routes'][0]['destination'])

    @decorators.idempotent_id('c9ad566e-fe8f-4559-8303-bbad9062a30c')
    def test_get_advertised_routes_tenant_networks(self):
        self.useFixture(fixtures.LockFixture('gateway_network_binding'))
        addr_scope = self.create_address_scope('my-scope', ip_version=4)
        ext_net = self.create_shared_network(**{'router:external': True})
        tenant_net = self.create_network()
        ext_subnetpool = self.create_subnetpool(
                                            'test-pool-ext',
                                            is_admin=True,
                                            default_prefixlen=24,
                                            address_scope_id=addr_scope['id'],
                                            prefixes=['8.0.0.0/8'])
        tenant_subnetpool = self.create_subnetpool(
                                            'tenant-test-pool',
                                            default_prefixlen=25,
                                            address_scope_id=addr_scope['id'],
                                            prefixes=['10.10.0.0/16'])
        self.create_subnet({'id': ext_net['id']},
                           cidr=netaddr.IPNetwork('8.0.0.0/24'),
                           ip_version=4,
                           client=self.admin_client,
                           subnetpool_id=ext_subnetpool['id'])
        tenant_subnet = self.create_subnet(
                                       {'id': tenant_net['id']},
                                       cidr=netaddr.IPNetwork('10.10.0.0/24'),
                                       ip_version=4,
                                       subnetpool_id=tenant_subnetpool['id'])
        ext_gw_info = {'network_id': ext_net['id']}
        router = self.create_router(ext_gw_info)
        self.admin_routers.append(router)
        self.admin_client.add_router_interface_with_subnet_id(
                                                       router['id'],
                                                       tenant_subnet['id'])
        self.admin_routerports.append({'router_id': router['id'],
                                       'subnet_id': tenant_subnet['id']})
        bgp_speaker = self.create_bgp_speaker(**self.default_bgp_speaker_args)
        bgp_speaker_id = bgp_speaker['id']
        self.bgp_adm_client.add_bgp_gateway_network(bgp_speaker_id,
                                                    ext_net['id'])
        routes = self.bgp_adm_client.get_bgp_advertised_routes(bgp_speaker_id)
        self.assertEqual(1, len(routes['advertised_routes']))
        self.assertEqual(tenant_subnet['cidr'],
                         routes['advertised_routes'][0]['destination'])
        fixed_ip = router['external_gateway_info']['external_fixed_ips'][0]
        self.assertEqual(fixed_ip['ip_address'],
                         routes['advertised_routes'][0]['next_hop'])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import clients as tempest_clients
from tempest.lib.services import clients
from tempest.lib.services.compute import availability_zone_client
from tempest.lib.services.compute import hypervisor_client
from tempest.lib.services.compute import interfaces_client
from tempest.lib.services.compute import keypairs_client
from tempest.lib.services.compute import servers_client
from tempest.lib.services.identity.v2 import tenants_client
from tempest.lib.services.identity.v3 import projects_client
from tempest.lib.services.network import qos_limit_bandwidth_rules_client
from tempest.lib.services.network import qos_minimum_bandwidth_rules_client
from tempest.lib.services.network import qos_minimum_packet_rate_rules_client

from neutron_tempest_plugin import config
from neutron_tempest_plugin.services.network.json import network_client

CONF = config.CONF


class Manager(clients.ServiceClients):
    """Top level manager for OpenStack tempest clients"""
    default_params = {
        'disable_ssl_certificate_validation':
            CONF.identity.disable_ssl_certificate_validation,
        'ca_certs': CONF.identity.ca_certificates_file,
        'trace_requests': CONF.debug.trace_requests,
        'proxy_url': CONF.service_clients.proxy_url
    }

    # NOTE: Tempest uses timeout values of compute API if project specific
    # timeout values don't exist.
    default_params_with_timeout_values = {
        'build_interval': CONF.compute.build_interval,
        'build_timeout': CONF.compute.build_timeout
    }
    default_params_with_timeout_values.update(default_params)

    def __init__(self, credentials=None, service=None):
        dscv = CONF.identity.disable_ssl_certificate_validation
        _, uri = tempest_clients.get_auth_provider_class(credentials)
        super().__init__(
            credentials=credentials,
            identity_uri=uri,
            scope='project',
            disable_ssl_certificate_validation=dscv,
            ca_certs=CONF.identity.ca_certificates_file,
            trace_requests=CONF.debug.trace_requests)

        self._set_identity_clients()

        self.network_client = network_client.NetworkClientJSON(
            self.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **self.default_params)

        params = {
            'service': CONF.compute.catalog_type,
            'region': CONF.compute.region or CONF.identity.region,
            'endpoint_type': CONF.compute.endpoint_type,
            'build_interval': CONF.compute.build_interval,
            'build_timeout': CONF.compute.build_timeout
        }
        params.update(self.default_params)

        self.servers_client = servers_client.ServersClient(
            self.auth_provider,
            enable_instance_password=CONF.compute_feature_enabled
            .enable_instance_password,
            **params)
        self.interfaces_client = interfaces_client.InterfacesClient(
            self.auth_provider, **params)
        self.keypairs_client = keypairs_client.KeyPairsClient(
            self.auth_provider, ssh_key_type=CONF.validation.ssh_key_type,
            **params)
        self.hv_client = hypervisor_client.HypervisorClient(
            self.auth_provider, **params)
        self.az_client = availability_zone_client.AvailabilityZoneClient(
            self.auth_provider, **params)

        self.qos_limit_bandwidth_rules_client = \
            qos_limit_bandwidth_rules_client.QosLimitBandwidthRulesClient(
                self.auth_provider,
                CONF.network.catalog_type,
                CONF.network.region or CONF.identity.region,
                endpoint_type=CONF.network.endpoint_type,
                build_interval=CONF.network.build_interval,
                build_timeout=CONF.network.build_timeout,
                **self.default_params)

        self.qos_minimum_bandwidth_rules_client = \
            qos_minimum_bandwidth_rules_client.QosMinimumBandwidthRulesClient(
                self.auth_provider,
                CONF.network.catalog_type,
                CONF.network.region or CONF.identity.region,
                endpoint_type=CONF.network.endpoint_type,
                build_interval=CONF.network.build_interval,
                build_timeout=CONF.network.build_timeout,
                **self.default_params)

        self.qos_minimum_packet_rate_rules_client = \
            qos_minimum_packet_rate_rules_client.\
            QosMinimumPacketRateRulesClient(
                self.auth_provider,
                CONF.network.catalog_type,
                CONF.network.region or CONF.identity.region,
                endpoint_type=CONF.network.endpoint_type,
                build_interval=CONF.network.build_interval,
                build_timeout=CONF.network.build_timeout,
                **self.default_params)

    def _set_identity_clients(self):
        params = {
            'service': CONF.identity.catalog_type,
            'region': CONF.identity.region
        }
        params.update(self.default_params_with_timeout_values)
        params_v2_admin = params.copy()
        params_v2_admin['endpoint_type'] = CONF.identity.v2_admin_endpoint_type
        # Client uses admin endpoint type of Keystone API v2
        self.tenants_client = tenants_client.TenantsClient(self.auth_provider,
                                                           **params_v2_admin)
        # Client uses admin endpoint type of Keystone API v3
        self.projects_client = projects_client.ProjectsClient(
            self.auth_provider, **params_v2_admin)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base


class TagTestJSON(base.BaseAdminNetworkTest):

    required_extensions = ['standard-attr-tag']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.res_id = cls._create_resource()

    def _get_and_compare_tags(self, tags, client):
        res_body = client.get_tags(self.resource, self.res_id)
        self.assertCountEqual(tags, res_body['tags'])

    def _test_tag_operations(self, client=None):
        client = client or self.client
        # create and get tags
        tags = ['red', 'blue']
        res_body = client.update_tags(self.resource, self.res_id, tags)
        self.assertCountEqual(tags, res_body['tags'])
        self._get_and_compare_tags(tags, client)

        # add a tag
        client.update_tag(self.resource, self.res_id, 'green')
        self._get_and_compare_tags(['red', 'blue', 'green'], client)

        # update tag exist
        client.update_tag(self.resource, self.res_id, 'red')
        self._get_and_compare_tags(['red', 'blue', 'green'], client)

        # add a tag with a dot
        client.update_tag(self.resource, self.res_id, 'black.or.white')
        self._get_and_compare_tags(['red', 'blue', 'green', 'black.or.white'],
                                   client)

        # replace tags
        tags = ['red', 'yellow', 'purple']
        res_body = client.update_tags(self.resource, self.res_id, tags)
        self.assertCountEqual(tags, res_body['tags'])
        self._get_and_compare_tags(tags, client)

        # get tag
        client.get_tag(self.resource, self.res_id, 'red')

        # get tag not exist
        self.assertRaises(lib_exc.NotFound, client.get_tag,
                          self.resource, self.res_id, 'green')

        # delete tag
        client.delete_tag(self.resource, self.res_id, 'red')
        self._get_and_compare_tags(['yellow', 'purple'], client)

        # delete tag not exist
        self.assertRaises(lib_exc.NotFound, client.delete_tag,
                          self.resource, self.res_id, 'green')

        # delete tags
        client.delete_tags(self.resource, self.res_id)
        self._get_and_compare_tags([], client)


class TagNetworkTestJSON(TagTestJSON):
    resource = 'networks'

    @classmethod
    def _create_resource(cls):
        network = cls.create_network()
        return network['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('5621062d-fbfb-4437-9d69-138c78ea4188')
    def test_network_tags(self):
        self._test_tag_operations()


class TagSubnetTestJSON(TagTestJSON):
    resource = 'subnets'

    @classmethod
    def _create_resource(cls):
        network = cls.create_network()
        subnet = cls.create_subnet(network)
        return subnet['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2805aabf-a94c-4e70-a0b2-9814f06beb03')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_subnet_tags(self):
        self._test_tag_operations()


class TagPortTestJSON(TagTestJSON):
    resource = 'ports'

    @classmethod
    def _create_resource(cls):
        network = cls.create_network()
        port = cls.create_port(network)
        return port['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c7c44f2c-edb0-4ebd-a386-d37cec155c34')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_port_tags(self):
        self._test_tag_operations()


class TagSubnetPoolTestJSON(TagTestJSON):
    resource = 'subnetpools'

    @classmethod
    @utils.requires_ext(extension="subnet_allocation", service="network")
    def _create_resource(cls):
        subnetpool = cls.create_subnetpool('subnetpool', default_prefixlen=24,
                                           prefixes=['10.0.0.0/8'])
        return subnetpool['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('bdc1c24b-c0b5-4835-953c-8f67dc11edfe')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_subnetpool_tags(self):
        self._test_tag_operations()


class TagRouterTestJSON(TagTestJSON):
    resource = 'routers'

    @classmethod
    @utils.requires_ext(extension="router", service="network")
    def _create_resource(cls):
        router = cls.create_router(router_name='test')
        return router['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('b898ff92-dc33-4232-8ab9-2c6158c80d28')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_router_tags(self):
        self._test_tag_operations()


class TagSecGroupTestJSON(TagTestJSON):
    resource = 'security-groups'

    @classmethod
    @utils.requires_ext(extension="security-group", service="network")
    def _create_resource(cls):
        sec_group = cls.create_security_group(name='test')
        return sec_group['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('0f1a78eb-c5be-42cf-919d-2ce3621a51c2')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_security_group_tags(self):
        self._test_tag_operations()


class TagFloatingIpTestJSON(TagTestJSON):
    resource = 'floatingips'

    @classmethod
    @utils.requires_ext(extension="router", service="network")
    def _create_resource(cls):
        return cls.create_floatingip()['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('53f6c2bf-e272-4e9e-b9a9-b165eb7be807')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_floatingip_tags(self):
        self._test_tag_operations()


class TagQosPolicyTestJSON(TagTestJSON):
    resource = 'policies'

    @classmethod
    @utils.requires_ext(extension="qos", service="network")
    def _create_resource(cls):
        qos_policy = cls.create_qos_policy(name='test-policy', shared=True)
        return qos_policy['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('e9bac15e-c8bc-4317-8295-4bf1d8d522b8')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_qos_policy_tags(self):
        self._test_tag_operations(client=self.admin_client)


class TagTrunkTestJSON(TagTestJSON):
    resource = 'trunks'

    @classmethod
    @utils.requires_ext(extension="trunk", service="network")
    def _create_resource(cls):
        network = cls.create_network()
        parent_port = cls.create_port(network)
        return cls.create_trunk(parent_port)['id']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('4c63708b-c4c3-407c-8101-7a9593882f5f')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_trunk_tags(self):
        self._test_tag_operations()


class TagFilterTestJSON(base.BaseAdminNetworkTest):
    credentials = ['primary', 'alt', 'admin']
    required_extensions = ['standard-attr-tag']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        try:
            client = cls.tag_client
        except AttributeError:
            client = cls.client

        cls.res_ids = []
        for i in range(5):
            cls.res_ids.append(cls._create_resource())

        client.update_tags(cls.resource, cls.res_ids[0], ['red'])
        client.update_tags(cls.resource, cls.res_ids[1], ['red', 'blue'])
        client.update_tags(cls.resource, cls.res_ids[2],
                           ['red', 'blue', 'green'])
        client.update_tags(cls.resource, cls.res_ids[3], ['green'])
        # 5th resource: no tags

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_alt.network_client

    def _assertEqualResources(self, expected, res):
        expected = [self.res_ids[i] for i in expected]
        actual = [n['id'] for n in res if n['id'] in self.res_ids]
        self.assertEqual(set(expected), set(actual))

    def _test_filter_tags(self):
        # tags single
        filters = {'tags': 'red'}
        res = self._list_resource(filters)
        self._assertEqualResources([0, 1, 2], res)

        # tags multi
        filters = {'tags': 'red,blue'}
        res = self._list_resource(filters)
        self._assertEqualResources([1, 2], res)

        # tags-any single
        filters = {'tags-any': 'blue'}
        res = self._list_resource(filters)
        self._assertEqualResources([1, 2], res)

        # tags-any multi
        filters = {'tags-any': 'red,blue'}
        res = self._list_resource(filters)
        self._assertEqualResources([0, 1, 2], res)

        # not-tags single
        filters = {'not-tags': 'red'}
        res = self._list_resource(filters)
        self._assertEqualResources([3, 4], res)

        # not-tags multi
        filters = {'not-tags': 'red,blue'}
        res = self._list_resource(filters)
        self._assertEqualResources([0, 3, 4], res)

        # not-tags-any single
        filters = {'not-tags-any': 'blue'}
        res = self._list_resource(filters)
        self._assertEqualResources([0, 3, 4], res)

        # not-tags-any multi
        filters = {'not-tags-any': 'red,blue'}
        res = self._list_resource(filters)
        self._assertEqualResources([3, 4], res)


class TagFilterNetworkTestJSON(TagFilterTestJSON):
    resource = 'networks'

    @classmethod
    def _create_resource(cls):
        res = cls.create_network()
        return res['id']

    def _list_resource(self, filters):
        res = self.client.list_networks(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('a66b5cca-7db2-40f5-a33d-8ac9f864e53e')
    def test_filter_network_tags(self):
        self._test_filter_tags()


class TagFilterSubnetTestJSON(TagFilterTestJSON):
    resource = 'subnets'

    @classmethod
    def _create_resource(cls):
        network = cls.create_network()
        res = cls.create_subnet(network)
        return res['id']

    def _list_resource(self, filters):
        res = self.client.list_subnets(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('dd8f9ba7-bcf6-496f-bead-714bd3daac10')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_subnet_tags(self):
        self._test_filter_tags()


class TagFilterPortTestJSON(TagFilterTestJSON):
    resource = 'ports'

    @classmethod
    def _create_resource(cls):
        network = cls.create_network()
        res = cls.create_port(network)
        return res['id']

    def _list_resource(self, filters):
        res = self.client.list_ports(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('09c036b8-c8d0-4bee-b776-7f4601512898')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_port_tags(self):
        self._test_filter_tags()


class TagFilterSubnetpoolTestJSON(TagFilterTestJSON):
    resource = 'subnetpools'

    @classmethod
    @utils.requires_ext(extension="subnet_allocation", service="network")
    def _create_resource(cls):
        res = cls.create_subnetpool('subnetpool', default_prefixlen=24,
                                    prefixes=['10.0.0.0/8'])
        return res['id']

    def _list_resource(self, filters):
        res = self.client.list_subnetpools(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('16ae7ad2-55c2-4821-9195-bfd04ab245b7')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_subnetpool_tags(self):
        self._test_filter_tags()


class TagFilterRouterTestJSON(TagFilterTestJSON):
    resource = 'routers'

    @classmethod
    @utils.requires_ext(extension="router", service="network")
    def _create_resource(cls):
        res = cls.create_router(router_name='test')
        return res['id']

    def _list_resource(self, filters):
        res = self.client.list_routers(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('cdd3f3ea-073d-4435-a6cb-826a4064193d')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_router_tags(self):
        self._test_filter_tags()


class TagFilterSecGroupTestJSON(TagFilterTestJSON):
    resource = 'security-groups'

    @classmethod
    @utils.requires_ext(extension="security-group", service="network")
    def _create_resource(cls):
        sec_group = cls.create_security_group(name='test')
        return sec_group['id']

    def _list_resource(self, filters):
        res = self.client.list_security_groups(**filters)
        resource_key = self.resource.replace('-', '_')
        return res[resource_key]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('d4d1d681-0116-4800-9725-16cb88f8171a')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_security_group_tags(self):
        self._test_filter_tags()


class TagFilterFloatingIpTestJSON(TagFilterTestJSON):
    resource = 'floatingips'

    @classmethod
    @utils.requires_ext(extension="router", service="network")
    def _create_resource(cls):
        return cls.create_floatingip()['id']

    def _list_resource(self, filters):
        res = self.client.list_floatingips(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('01f00afc-dbec-432a-bfee-2a1f0510e7a8')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_floatingip_tags(self):
        self._test_filter_tags()


class TagFilterQosPolicyTestJSON(TagFilterTestJSON):
    resource = 'policies'

    @classmethod
    @utils.requires_ext(extension="qos", service="network")
    def _create_resource(cls):
        qos_policy = cls.create_qos_policy(name='test-policy', shared=True)
        return qos_policy['id']

    def _list_resource(self, filters):
        res = self.client.list_qos_policies(**filters)
        return res[self.resource]

    @classmethod
    def resource_setup(cls):
        cls.tag_client = cls.admin_client
        super().resource_setup()

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c2f9a6ae-2529-4cb9-a44b-b16f8ba27832')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_qos_policy_tags(self):
        self._test_filter_tags()


class TagFilterTrunkTestJSON(TagFilterTestJSON):
    resource = 'trunks'

    @classmethod
    @utils.requires_ext(extension="trunk", service="network")
    def _create_resource(cls):
        network = cls.create_network()
        parent_port = cls.create_port(network)
        return cls.create_trunk(parent_port)['id']

    def _list_resource(self, filters):
        res = self.client.list_trunks(**filters)
        return res[self.resource]

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('3fb3ca3a-8e3a-4565-ba73-16413d445e25')
    @utils.requires_ext(extension="standard-attr-tag", service="network")
    def test_filter_trunk_tags(self):
        self._test_filter_tags()


class UpdateTagsTest(base.BaseAdminNetworkTest):

    required_extensions = ['standard-attr-tag']

    def _get_and_compare_tags(self, tags, res_id):
        # nothing specific about networks here, just a resource that is
        # available in all setups
        res_body = self.client.get_tags('networks', res_id)
        self.assertCountEqual(tags, res_body['tags'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('74c56fb1-a3b1-4a62-a8d2-d04dca6bd4cd')
    def test_update_tags_affects_only_updated_resource(self):
        res1 = self.create_network()
        res2 = self.create_network()

        self.client.update_tags('networks', res1['id'], ['red', 'blue'])
        self._get_and_compare_tags(['red', 'blue'], res1['id'])

        self.client.update_tags('networks', res2['id'], ['red'])
        self._get_and_compare_tags(['red'], res2['id'])

        self.client.update_tags('networks', res2['id'], [])
        self._get_and_compare_tags([], res2['id'])

        # check that updates on res2 hasn't dropped tags from res1
        self._get_and_compare_tags(['red', 'blue'], res1['id'])



#   Copyright 2021 Huawei, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from oslo_log import log as logging
from tempest.common import utils
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import constants as const

LOG = logging.getLogger(__name__)
CONF = config.CONF


class LocalIPTest(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']

    @classmethod
    @utils.requires_ext(extension="local_ip", service="network")
    def resource_setup(cls):
        super().resource_setup()
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.keypair = cls.create_keypair()

        # Create security group with admin privileges
        cls.secgroup = cls.create_security_group(
            name=data_utils.rand_name('secgroup'))

        # Execute funcs to achieve ssh and ICMP capabilities
        cls.create_loginable_secgroup_rule(secgroup_id=cls.secgroup['id'])
        cls.create_pingable_secgroup_rule(secgroup_id=cls.secgroup['id'])

        # Create router
        cls.router = cls.create_router(
            router_name=data_utils.rand_name("router-test"),
            admin_state_up=True,
            external_network_id=CONF.network.public_network_id)
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])

    def _create_server(self, name=None):
        port = self.create_port(
            self.network, security_groups=[self.secgroup['id']])
        server = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'], name=name,
            networks=[{'port': port['id']}])['server']
        waiters.wait_for_server_status(self.os_primary.servers_client,
                                       server['id'],
                                       const.SERVER_STATUS_ACTIVE)

        return {'port': port, 'server': server}

    @decorators.idempotent_id('3aa4b288-011a-4aa2-9024-19ad2ce40bfd')
    def test_local_ip_connectivity(self):
        server1 = self._create_server(name='local_ip_vm1')
        server2 = self._create_server(name='local_ip_vm2')

        fip = self.create_and_associate_floatingip(server1['port']['id'])
        ssh_client = ssh.Client(
            fip['floating_ip_address'],
            CONF.validation.image_ssh_user,
            pkey=self.keypair['private_key'])

        servers = [server1['server'], server2['server']]

        # first check basic connectivity
        self.check_remote_connectivity(
            ssh_client,
            server2['port']['fixed_ips'][0]['ip_address'],
            servers=servers)

        local_ip = self.create_local_ip(network_id=self.network['id'])
        self.create_local_ip_association(local_ip['id'],
                                         fixed_port_id=server2['port']['id'])
        # check connectivity with local ip address
        self.check_remote_connectivity(
            ssh_client, local_ip['local_ip_address'],
            servers=servers, check_response_ip=False)

        # check basic connectivity after local ip association
        self.check_remote_connectivity(
            ssh_client,
            server2['port']['fixed_ips'][0]['ip_address'],
            servers=servers,
            check_response_ip=False)



# Copyright (c) 2019 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config

CONF = config.CONF


class ConntrackHelperTestJSON(base.BaseNetworkTest):

    required_extensions = ['router', 'l3-conntrack-helper',
                           'expose-l3-conntrack-helper']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.ext_net_id = CONF.network.public_network_id

        # Create network, subnet
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)

    @decorators.idempotent_id('6361c80e-902d-4c2a-88b4-ea8066507eee')
    def test_create_list_update_show_delete_conntrack_helper(self):
        # Create a router
        router = self.create_router(data_utils.rand_name('router'),
                                    external_network_id=self.ext_net_id)

        # Create conntrack helper
        created_cth = self.create_conntrack_helper(router['id'], helper='tftp',
                                                   protocol='udp', port=69)
        self.assertEqual('tftp', created_cth['helper'])
        self.assertEqual('udp', created_cth['protocol'])
        self.assertEqual(69, created_cth['port'])

        # List conntrack helpers
        conntrack_helpers = self.client.list_conntrack_helpers(
            router['id'])['conntrack_helpers']
        self.assertIn(created_cth['id'],
                      {cth['id'] for cth in conntrack_helpers})

        # Update conntrack helper
        updated_conntrack_helper = self.client.update_conntrack_helper(
            router['id'], created_cth['id'], port=6969)['conntrack_helper']
        self.assertEqual(updated_conntrack_helper['port'], 6969)

        # Show conntrack helper
        conntrack_helper = self.client.get_conntrack_helper(
            router['id'], created_cth['id'])['conntrack_helper']
        self.assertEqual('tftp', conntrack_helper['helper'])
        self.assertEqual('udp', conntrack_helper['protocol'])
        self.assertEqual(6969, conntrack_helper['port'])

        # Delete conntrack helper
        self.client.delete_conntrack_helper(router['id'], created_cth['id'])
        self.assertRaises(
            exceptions.NotFound,
            self.client.get_conntrack_helper, router['id'], created_cth['id'])

    @decorators.idempotent_id('0a6ae20c-3f66-423e-93c6-cfedd1c93b8d')
    def test_conntrack_helper_info_in_router_details(self):
        # Create a router
        router = self.create_router(data_utils.rand_name('router'),
                                    external_network_id=self.ext_net_id)

        # Ensure routerd does not have information about any conntrack helper
        router = self.client.show_router(router['id'])['router']
        self.assertEqual(0, len(router['conntrack_helpers']))

        # Now create conntrack helper and ensure it's visible in Router details
        cth = self.create_conntrack_helper(router['id'], helper='ftp',
                                           protocol='tcp', port=21)
        router = self.client.show_router(router['id'])['router']
        self.assertEqual(1, len(router['conntrack_helpers']))
        self.assertEqual('ftp', router['conntrack_helpers'][0]['helper'])
        self.assertEqual('tcp', router['conntrack_helpers'][0]['protocol'])
        self.assertEqual(21, router['conntrack_helpers'][0]['port'])

        # Delete conntrack_helper and ensure it's no longer in Router details
        self.client.delete_conntrack_helper(router['id'], cth['id'])
        router = self.client.show_router(router['id'])['router']
        self.assertEqual(0, len(router['conntrack_helpers']))

    @decorators.idempotent_id('134469d9-fb25-4165-adc8-f4747f07caf1')
    def test_2_conntrack_helpers_to_same_router(self):
        # Create a router
        router = self.create_router(data_utils.rand_name('router'),
                                    external_network_id=self.ext_net_id)

        cth_data = [{'helper': 'tftp', 'protocol': 'udp', 'port': 60},
                    {'helper': 'ftp', 'protocol': 'tcp', 'port': 21}]
        created_cths = []
        for cth in cth_data:
            created_cth = self.create_conntrack_helper(
                router_id=router['id'],
                helper=cth['helper'],
                protocol=cth['protocol'],
                port=cth['port'])
            self.assertEqual(cth['helper'], created_cth['helper'])
            self.assertEqual(cth['protocol'], created_cth['protocol'])
            self.assertEqual(cth['port'], created_cth['port'])
            created_cths.append(created_cth)

        # Check that conntrack helpers are in Router details
        router = self.client.show_router(router['id'])['router']
        self.assertEqual(len(cth_data), len(router['conntrack_helpers']))
        for cth in created_cths:
            expected_cth = cth.copy()
            expected_cth.pop('id')
            expected_cth.pop('client')
            expected_cth.pop('router_id')
            self.assertIn(expected_cth, router['conntrack_helpers'])

        # Test list of conntrack helpers
        conntrack_helpers = self.client.list_conntrack_helpers(
            router['id'])['conntrack_helpers']
        self.assertEqual(len(cth_data), len(conntrack_helpers))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from oslo_utils import uuidutils
from tempest.common import utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config


class NetworksTestAdmin(base.BaseAdminNetworkTest):

    @decorators.idempotent_id('d3c76044-d067-4cb0-ae47-8cdd875c7f67')
    @utils.requires_ext(extension="project-id", service="network")
    def test_create_network_with_project(self):
        project_id = self.client.project_id  # non-admin

        name = 'admin-created-with-project_id'
        network = self.create_network(name, project_id=project_id,
                                      client=self.admin_client)
        self.assertEqual(name, network['name'])
        self.assertEqual(project_id, network['project_id'])
        self.assertEqual(project_id, network['tenant_id'])

        observed_network = self.client.list_networks(
            id=network['id'])['networks'][0]
        self.assertEqual(name, observed_network['name'])
        self.assertEqual(project_id, observed_network['project_id'])
        self.assertEqual(project_id, observed_network['tenant_id'])

    @decorators.idempotent_id('8d21aaca-4364-4eb9-8b79-44b4fff6373b')
    @utils.requires_ext(extension="project-id", service="network")
    def test_create_network_with_project_and_tenant(self):
        project_id = self.client.project_id  # non-admin

        name = 'created-with-project-and-tenant'
        network = self.create_network(name, project_id=project_id,
                                      tenant_id=project_id,
                                      client=self.admin_client)
        self.assertEqual(name, network['name'])
        self.assertEqual(project_id, network['project_id'])
        self.assertEqual(project_id, network['tenant_id'])

        observed_network = self.client.list_networks(
            id=network['id'])['networks'][0]
        self.assertEqual(name, observed_network['name'])
        self.assertEqual(project_id, observed_network['project_id'])
        self.assertEqual(project_id, observed_network['tenant_id'])

    @decorators.idempotent_id('08b92179-669d-45ee-8233-ef6611190809')
    @utils.requires_ext(extension="project-id", service="network")
    def test_create_network_with_project_and_other_tenant(self):
        project_id = self.client.project_id  # non-admin
        other_tenant = uuidutils.generate_uuid()

        name = 'created-with-project-and-other-tenant'
        e = self.assertRaises(lib_exc.BadRequest,
                              self.create_network, name,
                              project_id=project_id, tenant_id=other_tenant,
                              client=self.admin_client)
        expected_message = "'project_id' and 'tenant_id' do not match"
        self.assertEqual(expected_message, e.resp_body['message'])

    @decorators.idempotent_id('571d0dde-0f84-11e7-b565-fa163e4fa634')
    @testtools.skipUnless("vxlan" in config.CONF.neutron_plugin_options.
                          available_type_drivers,
                          'VXLAN type_driver is not enabled')
    @utils.requires_ext(extension="provider", service="network")
    def test_create_tenant_network_vxlan(self):
        network = self.admin_client.create_network(
            **{"provider:network_type": "vxlan"})['network']
        self.addCleanup(self.admin_client.delete_network,
                        network['id'])
        network = self.admin_client.show_network(
            network['id'])['network']
        self.assertEqual('vxlan', network['provider:network_type'])

    @decorators.idempotent_id('bbb9a2be-c9a7-4693-ac8e-d51b5371b68d')
    def test_list_network_filter_provider_attributes(self):
        if not config.CONF.neutron_plugin_options.provider_vlans:
            raise self.skipException("No provider VLAN networks available")
        project_id = self.client.project_id
        physnet_name = config.CONF.neutron_plugin_options.provider_vlans[0]
        # Check project networks pre-created.
        body = self.client.list_networks(project_id=project_id)['networks']
        num_networks_precreated = len(body)

        networks = []
        num_networks = 5
        for _ in range(num_networks):
            networks.append(self.create_network(
                provider_network_type='vlan',
                provider_physical_network=physnet_name,
                project_id=project_id))

        # Check new project networks created.
        body = self.client.list_networks(project_id=project_id)['networks']
        self.assertEqual(num_networks + num_networks_precreated, len(body))

        vlan_ids = [net['provider:segmentation_id'] for net in networks]

        # List networks with limit (from 1 to num_networks).
        # Each filter (except from the 'provider:segmentation_id'), uses the
        # value directly and in a list.
        for idx in range(1, num_networks + 1):
            # Filter by 'provider:network_type'
            kwargs = {'provider:network_type': 'vlan',
                      'project_id': project_id, 'limit': idx}
            body = self.client.list_networks(**kwargs)['networks']
            self.assertEqual(idx, len(body))
            kwargs['provider:network_type'] = ['vlan']
            body = self.client.list_networks(**kwargs)['networks']
            self.assertEqual(idx, len(body))

            # Filter by 'provider:physical_network'.
            kwargs = {'provider:physical_network': physnet_name,
                      'project_id': project_id, 'limit': idx}
            body = self.client.list_networks(**kwargs)['networks']
            self.assertEqual(idx, len(body))
            kwargs['provider:physical_network'] = [physnet_name]
            body = self.client.list_networks(**kwargs)['networks']
            self.assertEqual(idx, len(body))

            # Filter by 'provider:segmentation_id'
            kwargs = {'provider:segmentation_id': vlan_ids,
                      'project_id': project_id, 'limit': idx}
            body = self.client.list_networks(**kwargs)['networks']
            self.assertEqual(idx, len(body))
